/* 
 * Process data from the DCT2000 and fill data structures accordingly.
 * 
 * Based highly on DCTTest by Don Starr
 * http://www.starrsoft.com/freeware/DCTTest.zip
 *
 * Heavily modifed by Dave Johnson <dave-dctremote@davej.org>
 */ 


#ifndef PACKET_H
#define PACKET_H

#include <stdint.h>

typedef enum
{
	_PT_ChannelStatus	= 0x20,
	_PT_ChannelStatusEx	= 0xa1,
	_PT_ChannelStatusReq	= 0x21,
	_PT_IRKeypress		= 0x22
} _eKnownPacketTypes;

typedef enum
{
	_SID_DCT	= 0x04,
	_SID_Sidecar	= 0x40
} _eSenderID;

typedef enum
{
	state_WaitPacketType,
	state_WaitLenHi,
	state_WaitLenLo,
	state_WaitSeq,
	state_WaitID,
	state_WaitPayload,
	state_WaitCRCHi,
	state_WaitCRCLo,
	state_WaitTerminator
} _eState;

typedef struct
{
	uint8_t		cPacketType;
	uint16_t	iLen;
	uint8_t		cSeq;
	uint8_t		cID;
	uint8_t		*cpPayload;
	uint16_t	iCRC;
} _sPacket;

typedef struct
{
	_eState		state;
	_sPacket	packet;
	int		payloadOffset;
	int		escReceived;
} _sProcessorInfo;

_sProcessorInfo *ProcessorInit( void );
void ProcessorCleanup( _sProcessorInfo *info );
int ProcessByte( _sProcessorInfo *info, unsigned char ch );

void packet_cleanup(_sPacket *p);
void packet_build_channelquery(_sPacket *p);
void packet_build_initialize_1(_sPacket *p);
void packet_build_initialize_2(_sPacket *p);
void packet_build_keypress(_sPacket *p, uint8_t key);
void packet_received(_sPacket *p);
void packet_sent(_sPacket *p);

int packet_is_ack(_sPacket *p, _sPacket *ack);
int packet_extract_channel(_sPacket *p);

#endif

